
/******************************************************************************
* MODULE     : tm_file.gen.h
* DESCRIPTION: Loading and saving TeXmacs files
* COPYRIGHT  : (C) 1999  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include <tm_buffer.gen.h>
#include <file.gen.h>
#include <dir.gen.h>
#include <convert.gen.h>

#module code_tm_file
#import tm_buffer
#import file
#import dir
#import convert

/******************************************************************************
* Subroutines for loading and saving files
******************************************************************************/

static string
file_type (string file_name) {
  int i;
  for (i=N(file_name)-1; i>=0; i--)
    if (file_name[i]=='.') break;
  if ((i>0) && (i<N(file_name)-1)) {
    string r= file_name (i+1, N(file_name));
    while ((N(r)>0) && (r[N(r)-1]=='~')) r= r(0, N(r)-1);
    return r;
  }
  return "";
}

static string
file_format (string file_name) {
  string tp= file_type (file_name);
  if ((tp == "tm") || (tp == "ts") || (tp == "tp")) return "TeXmacs";
  if ((tp == "tex") || (tp == "sty") || (tp == "cls")) return "latex";
  if (tp == "html") return "html";
  else return "verbatim";
}

static bool
has_no_name (string s) {
  return (N(s)>=7) && (s(0,7)=="no name");
}

/******************************************************************************
* Loading files
******************************************************************************/

tree
tm_data_rep::file_to_tree (string dir, string name, string fm) {
  string fnf   = "Error: file#" * name * "#not found";
  string action= "load " * fm * " file#" * name;

  if (!file_exists (dir, name)) { set_message (fnf, action); return "error"; }
  string unique_name= get_unique_file_name (dir, name);

  if ((fm == "TeXmacs") ||
      ((fm == "help") && (name (N(name)-3, N(name)) == ".tm"))) {
    tree doc;
    if (load_tree (dir, name, doc)) set_message (fnf, action);
    if (doc == "error") set_message ("Error: bad data format", action);
    return doc;
  }

  else if (fm == "scheme") {
    string s;
    if (load_string (dir, name, s)) set_message (fnf, action);
    else {
      tree doc= scheme_document_to_tree (s);
      if (doc == "error") set_message ("Error: bad data format", action);
      return doc;
    }
  }

  else if ((fm == "tex") || (fm == "latex") ||
	   ((fm == "help") && (name (N(name)-4, N(name)) == ".tex"))) {
    string s, style;
    if (load_string (dir, name, s)) set_message (fnf, action);
    else {
      tree t= simplify_correct (latex_document_to_tree (string (s), style));
      if (fm == "help") style= string ("help");
      return tree (DOCUMENT,
		   tree (EXPAND, "body", t),
		   tree (EXPAND, "style", style));
    }
  }

  else if ((fm == "html") ||
	   ((fm == "help") && (name (N(name)-5, N(name)) == ".html"))) {
    string s, style;
    if (load_string (dir, name, s)) set_message (fnf, action);
    else {
      tree t= simplify_correct (html_document_to_tree (string (s), style));
      if (fm == "help") style= string ("help");
      return tree (DOCUMENT,
		   tree (EXPAND, "body", t),
		   tree (EXPAND, "style", style));
    }
  }

  else if (fm == "verbatim") {
    string s;
    if (load_string (dir, name, s)) set_message (fnf, action);
    else {
      tree t    = verbatim_to_tree (s * "\n");
      tree init = tree (COLLECTION,
			tree (ASSOCIATE, TEXT_LANGUAGE, "verbatim"),
			tree (ASSOCIATE, TEXT_FAMILY, "tt"),
			tree (ASSOCIATE, PAR_FIRST, "0cm"));
      return tree (DOCUMENT,
		   tree (EXPAND, "body", t),
		   tree (EXPAND, "initial", init));
    }
  }

  else set_message ("Error: unknown format", action);
  return "error";
}

void
tm_data_rep::load_buffer (
  string dir, string name, string fm, int where, bool test_newer)
{
  if (!file_exists (dir, name)) {
    cerr << "TeXmacs] File '" << name << "' not found\n";
    if (N(bufs)!=0)
      set_message ("Error: file#'" * name * "'#not found",
		   "load " * fm * " file");
    return;
  }
  if (test_newer && file_exists (dir, name * "~")) {
    string unique_name= get_unique_file_name (dir, name);
    string backup_name= get_unique_file_name (dir, name * "~");
    if (is_newer (backup_name, unique_name)) {
      string args  =
	"\"" * dir * "\" " *
	"\"" * name * "\" " *
	"\"" * fm * "\" " *
	"\"" * as_string (where) * "\"";
      string lambda= "'(lambda (s) (recover-auto-save " * args * " s))";
      exec ("(interactive '(\"Load more recent autosave file?\") "*lambda*")");
      return;
    }
  }

  string unique_name= get_unique_file_name (dir, name);
  if ((!test_newer) && (unique_name [N(unique_name)-1] == '~'))
    unique_name= unique_name (0, N(unique_name)-1);
  int nr= find_buffer (unique_name);
  tree doc= ((nr == -1)? file_to_tree (dir, name, fm): tree (DOCUMENT));
  if (doc == "error") return;
  switch (where) {
  case 0: new_buffer_in_this_window (unique_name, doc); break;
  case 1: new_buffer_in_new_window (unique_name, doc); break;
  case 2: new_buffer (unique_name, doc); break;
  default: fatal_error ("Bad value for 'where'", "load_buffer");
  }
  nr= find_buffer (unique_name);
  if (nr != -1) {
    tm_buffer buf= (tm_buffer) bufs[nr];
    buf->fm= fm;
    if ((fm == "help") || is_web_file (unique_name)) {
      tm_buffer buf= get_buffer ();
      buf->read_only= TRUE;
    }
  }
}

void
tm_data_rep::load_buffer (
  string name, string fm, int where, bool test_newer)
{
  if (fm == "help") {
    string lan   = get_output_language ();
    string path_s=
      "$TEXMACS_DOC_PATH/" * lan * ":$TEXMACS_PATH/doc/" * lan *
      ":$TEXMACS_DOC_PATH/english" * ":$TEXMACS_PATH/doc/english";
    if (file_exists (path_s, name * ".tm"))
      load_buffer (path_s, name * ".tm", fm, where, test_newer);
    else if (file_exists (path_s, name * ".tex"))
      load_buffer (path_s, name * ".tex", fm, where, test_newer);
    else {
      cerr << "TeXmacs] Help file '" << name << "' not found\n";
      if (N(bufs)!=0)
	set_message ("Error: help file#'" * name * "'#not found",
		     "load help file");
    }
  }
  else load_buffer ("$TEXMACS_FILE_PATH", name, fm, where, test_newer);
}

void
tm_data_rep::load_generic_buffer (
  string dir, string name, int where, bool test_newer)
{
  if (has_no_name (name)) return;
  load_buffer (dir, name, file_format (name), where, test_newer);
}

void
tm_data_rep::load_generic_buffer (string name, int where, bool test_newer) {
  load_generic_buffer ("$TEXMACS_FILE_PATH", name, where, test_newer);
}

tm_buffer
tm_data_rep::load_passive_buffer (string name) {
  int nr= find_buffer (name);
  if (nr != -1) return (tm_buffer) bufs[nr];
  load_buffer (".", name, "TeXmacs", 2, FALSE);
  nr= find_buffer (name);
  if (nr != -1) return (tm_buffer) bufs[nr];
  return NULL;
}

/******************************************************************************
* Saving files
******************************************************************************/

tree
tm_data_rep::make_document (tm_view vw) {
  tree doc (DOCUMENT);
  doc << tree (EXPAND, "TeXmacs", TEXMACS_VERSION);
  if (vw->buf->project != "")
    doc << tree (EXPAND, "project", vw->buf->project);
  if (vw->ed->get_style() != tree (TUPLE))
    doc << tree (EXPAND, "style", copy (vw->ed->get_style()));
  if (vw->buf->t != tree (DOCUMENT, ""))
    doc << tree (EXPAND, "body", vw->buf->t);
  if (N (vw->ed->get_init()) != 0)
    doc << tree (EXPAND, "initial", ((tree) vw->ed->get_init()));
  if (N (vw->ed->get_fin()) != 0)
    doc << tree (EXPAND, "final", ((tree) vw->ed->get_fin()));
  if (N (vw->buf->ref) != 0)
    doc << tree (EXPAND, "references", ((tree) vw->buf->ref));
  if (N (vw->buf->aux) != 0)
    doc << tree (EXPAND, "auxiliary", ((tree) vw->buf->aux));
  return doc;
}

bool
tm_data_rep::save (string name, tm_view vw) {
  tree doc= make_document (vw);
  bool flag= save_tree (name, doc);
  if (name == "~/.TeXmacs-save.tm") return flag;
  if (flag) set_message ("Error: file " * name * " did not open",
			 "save TeXmacs file");
  else set_message ("saved " * name, "save TeXmacs file");
  return flag;
}

void
tm_data_rep::save_buffer (string name, string fm) {
  string action= "save " * fm * " file";

  tm_buffer buf= get_buffer ();

  if (buf->read_only && ((fm == "TeXmacs") || (fm == "help"))) {
    set_message ("Error: file is read only", action);
    return;
  }

  if ((fm == "TeXmacs") && (!buf->needs_to_be_saved ()))
    if (buf->name == name) {
      set_message ("No changes need to be saved", action);
      return;
    }

  if (fm == "TeXmacs") {
    tm_view vw= get_view ();
    if (!save (name, vw)) {
      set_name_buffer (get_unique_file_name ("", name));
      pretend_save_buffer ();
    }
  }

  else if (fm == "scheme") {
    tm_view vw= get_view ();
    if (save_string (name, tree_to_scheme_document (make_document (vw))))
      set_message ("Error: file did not open", action);
    else set_message ("saved " * name, action);
  }

  else if ((fm == "tex") || (fm == "latex") || (fm == "help")) {
    editor ed   = get_editor ();
    tree   t    = ed->get_style();
    string style= is_atomic (t)? as_string (t): as_string (t[0]);
    string lan  = as_string (ed->get_init() [TEXT_LANGUAGE]);
    if (save_string (name, tree_to_latex_document (ed->et, style, lan)))
      set_message ("Error: file did not open", action);
    else set_message ("saved " * name, action);
  }

  else if (fm == "verbatim") {  
    if (save_string (name, tree_to_verbatim (get_editor()->et)))
      set_message ("Error: file did not open", action);
    else set_message ("saved " * name, action);
  }

  else set_message ("Error: unknown format", action);
}

void
tm_data_rep::save_buffer (string dir, string name, string fm) {
  save_buffer (get_unique_file_name (dir, name), fm);
}

void
tm_data_rep::save_generic_buffer (string name) {
  if (has_no_name (name)) return;
  save_buffer (name, file_format (name));
}

void
tm_data_rep::save_generic_buffer (string dir, string name) {
  save_generic_buffer (get_unique_file_name (dir, name));
}

/******************************************************************************
* Other routines concerning loading and saving buffers
******************************************************************************/

bool
tm_data_rep::no_name () {
  tm_buffer buf= get_buffer ();
  return has_no_name (buf->name);
}

bool
tm_data_rep::exists_unsaved_buffer () {
  bool flag= FALSE;
  int i, n= N(bufs);
  for (i=0; i<n; i++) {
    tm_buffer buf= (tm_buffer) bufs[i];
    flag = flag || buf->needs_to_be_saved ();
  }
  return flag;
}

void
tm_data_rep::pretend_save_buffer () {
  tm_buffer buf= get_buffer ();
  buf->mark_undo_block ();
  buf->need_save= buf->need_autosave= FALSE;
  buf->last_save= buf->last_autosave= buf->undo_depth- 1;
}

void
tm_data_rep::auto_save () {
  int i, n= N(bufs);
  for (i=0; i<n; i++) {
    tm_buffer buf= (tm_buffer) bufs[i];
    if ((buf->needs_to_be_autosaved () && (!buf->read_only))) {
      string name= buf->name * "~";
      if (has_no_name (buf->name))
	name= "$TEXMACS_HOME_PATH/system/autosave.tm";
      if (N(buf->vws)!=0) {
	if (!save (name, (tm_view) buf->vws[0])) {
	  buf->mark_undo_block ();
	  buf->need_autosave= FALSE;
	  buf->last_autosave= buf->undo_depth- 1;
	}
      }
    }
  }
  get_display()->delayed_message (get_meta()->get_this(), "auto save", 120000);
}

void
tm_data_rep::recover_auto_save (
  string dir, string name, string fm, string where, string answer)
{
  if (is_yes (answer))
    load_buffer (dir, name * "~", fm, as_int (where), FALSE);
  else load_buffer (dir, name, fm, as_int (where), FALSE);
}

#endmodule // code_tm_buffer
