
/******************************************************************************
* MODULE     : texmacs.gen.cc
* DESCRIPTION: main program
* COPYRIGHT  : (C) 1999  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include <dir.gen.h>
#include <server.gen.h>
#include <widget.gen.h>

#module code_texmacs
#import dir
#import server
#import widget

extern bool   char_clip;
extern bool   install_guile (int, char**, void (*) (int,char**));
extern string the_init_file;
extern string the_init_buffer_file;
extern int    install_status;
extern int    geometry_w, geometry_h;

/******************************************************************************
* Real main program for encaptulation of guile
******************************************************************************/

void
TeXmacs_main (int argc, char** argv) {
  int i;
  bool flag= TRUE;
  for (i=1; i<argc; i++)
    if (((argv[i][0] == '-') || (argv[i][0] == '+')) && (argv[i][1] != '\0')) {
      string s= argv[i];
      if ((s == "-s") || (s == "--silent")) flag= FALSE;
      else if ((s == "-e") || (s == "--events")) debug (1, TRUE);
      else if ((s == "-d") || (s == "--debug")) debug (2, TRUE);
      else if ((s == "-g") || (s == "--geometry")) {
	i++;
	if (i<argc) {
	  string g= argv[i];
	  int j;
	  for (j=0; j<N(g); j++)
	    if (g[j] == 'x') break;
	  if (j<N(g)) {
	    geometry_w= max (as_int (g (0, j)), 320);
	    geometry_h= max (as_int (g (j+1, N(g))), 200);
	  }
	}
      }
      else if ((s == "-b") || (s == "--initialize-buffer")) {
	i++;
	if (i<argc) the_init_buffer_file= argv[i];
      }
      else if ((s == "-i") || (s == "--initialize")) {
	i++;
	if (i<argc) the_init_file= argv[i];
      }
      else if ((s == "-v") || (s == "--version")) {
	cout << "\n";
	cout << "TeXmacs version " << TEXMACS_VERSION << "\n";
	cout << TEXMACS_COPYRIGHT << "\n";
	cout << "\n";
	exit (0);
      }
      else if ((s == "-p") || (s == "--path")) {
	system ("echo $TEXMACS_PATH");
	exit (0);
      }
      else if ((s == "-p") || (s == "--binpath")) {
	system ("echo $TEXMACS_BIN_PATH");
	exit (0);
      }
      else if ((s == "-x") || (s == "manual")) {
	system ("xdvi $TEXMACS_PATH/doc/dvi/texmacs.dvi &");
        exit (0);
      }
      else if ((s == "-Oc") || (s == "--no-char-clipping")) char_clip= FALSE;
      else if ((s == "+Oc") || (s == "--char-clipping")) char_clip= TRUE;
      else {
	cout << "\n";
	cout << "Options for TeXmacs:\n\n";
	cout << "  -b [file] Specify scheme buffers initialization file\n";
	cout << "  -d        For debugging purposes\n";
	cout << "  -e        Display all widget events\n";
	cout << "  -g [wxh]  Set window size to width x height\n";
	cout << "  -h        Display this help message\n";
	cout << "  -i [file] Specify scheme initialization file\n";
	cout << "  -p        Get the TeXmacs path\n";
	cout << "  -s        Do not inform about loaded files\n";
	cout << "  -v        Display current TeXmacs version\n";
	cout << "  -x        Consult the TeXmacs manual using xdvi\n";
	cout << "  -Oc       TeX characters bitmap clipping off\n";
	cout << "  +Oc       TeX characters bitmap clipping on (default)\n";
	cout << "\n";
	exit (0);
      }
    }
  if (flag) debug (0, TRUE);

  cout << "\n";
  if (debug (2)) cout << "TeXmacs] Installing TeX...\n";
  install_tex ();
  if (debug (2)) cout << "TeXmacs] Opening display...\n";
  display dis= open_display (argc, argv);
  if (debug (2)) cout << "TeXmacs] Starting server...\n";
  server sv (dis);

  for (i=1; i<argc; i++) {
    string s= argv[i];
    if ((s[0] != '-') && (s[0] != '+')) {
      if (debug (2)) cout << "TeXmacs] Loading " << s << "...\n";
      sv->load_generic_buffer (s, 1, FALSE);
    }
    if ((s == "-b") || (s == "--initialize-buffer") ||
	(s == "-i") || (s == "--initialize") ||
	(s == "-g") || (s == "--geometry")) i++;
  }
  if (install_status == 1) {
    if (debug (2)) cout << "TeXmacs] Loading welcome message...\n";
    sv->load_buffer ("startup/readme", "help", 1, FALSE);
  }
  else if (install_status == 2) {
    if (debug (2)) cout << "TeXmacs] Loading upgrade message...\n";
    sv->load_buffer ("startup/upgrade-last", "help", 1, FALSE);
  }
  if (sv->no_bufs ()) {
    if (debug (2)) cout << "TeXmacs] Creating 'no name' buffer...\n";
    sv->open_window ();
  }

  if (debug (2)) cout << "TeXmacs] Starting event loop...\n";
  dis->delayed_message (sv->get_meta(), "auto save", 120000);
  dis->delayed_message (sv->get_meta(), "banner", 100);
  dis->event_loop ();

  if (debug (2)) cout << "TeXmacs] Closing display...\n";
  close_display (dis);
  if (debug (2)) cout << "TeXmacs] Good bye...\n";
}

/******************************************************************************
* Main program
******************************************************************************/

int
main (int argc, char** argv) {
  install_texmacs ();
  install_guile (argc, argv, TeXmacs_main);
  return 0;
}

#endmodule // code_texmacs
