
/******************************************************************************
* MODULE     : pager.gen.cc
* DESCRIPTION: Control routines for typesetting paragraphs
* COPYRIGHT  : (C) 1999  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include <Page/pager.gen.h>

#module code_pager
#import pager

/******************************************************************************
* Routines for the pager class
******************************************************************************/

pager_rep::pager_rep (edit_env env2, array<page_item> l2):
  env (env2), style (UNINIT), l (l2)
{
  style (PAGE_ODD_HEADER) = env->read (PAGE_ODD_HEADER);
  style (PAGE_ODD_FOOTER) = env->read (PAGE_ODD_FOOTER);
  style (PAGE_EVEN_HEADER)= env->read (PAGE_EVEN_HEADER);
  style (PAGE_EVEN_FOOTER)= env->read (PAGE_EVEN_FOOTER);
  style (PAGE_THIS_HEADER)= "";
  style (PAGE_THIS_FOOTER)= "";

  SI dummy;
  paper   = (env->get_string (PAGE_MEDIUM) == "paper");
  env->get_page_pars (dummy, text_height, width, height, odd, even, top, bot);
  head_sep= env->get_length (PAGE_HEAD_SEP);
  foot_sep= env->get_length (PAGE_FOOT_SEP);
  show_hf = env->get_bool (PAGE_SHOW_HF) && paper;

  cur_top    = 0;
}

/******************************************************************************
* Typesetting a page
******************************************************************************/

void
pager_rep::start_page () {
  cur_spc = space (0);
  cur_sep = space (0);
  lines_bx= array<box> (0);
  lines_ht= array<space> (0);
}

void
pager_rep::print (page_item item) {
  cur_spc= cur_spc+ cur_sep+ space (item->b->h());
  cur_sep= item->spc;
  if (item->type == PAGE_CONTROL_ITEM) {
    if (is_tuple (item->t, "env_page")) {
      if (((item->t[1] == PAGE_THIS_HEADER) ||
	   (item->t[1] == PAGE_THIS_FOOTER)) &&
	   (item->t[2] == "")) style (item->t[1]->label)= " ";
      else style (item->t[1]->label)= copy (item->t[2]);
    }
    return;
  }
  lines_bx << item->b;
  lines_ht << item->spc;
}

void
pager_rep::end_page (bool flag) {
  int i;
  array<SI> spc (N(lines_bx));
  
  // stretching case
  if (paper && (!flag) && (cur_spc->def < text_height) &&
      (cur_spc->max > cur_spc->def)) {
    double f=
      ((double) (text_height - cur_spc->def)) /
      ((double) (cur_spc->max - cur_spc->def));
    for (i=0; i<N(lines_bx)-1; i++)
      spc[i]= lines_ht[i]->def+
	((SI) (f*((double) lines_ht[i]->max- lines_ht[i]->def)));
  }

  // shrinking case
  else if (paper && (cur_spc->def > text_height) &&
	   (cur_spc->def > cur_spc->min)) {
    double f=
      ((double) (cur_spc->def - text_height)) /
      ((double) (cur_spc->def - cur_spc->min));
    if (f>1.0) f=1.0;
    for (i=0; i<N(lines_bx)-1; i++)
      spc[i]= lines_ht[i]->def-
	((SI) (f*((double) lines_ht[i]->def- lines_ht[i]->min)));
  }

  // normal case
  else for (i=0; i<N(lines_bx)-1; i++) spc[i]= lines_ht[i]->def;

  box lb= move_box (path (), stack_box (path (), lines_bx, spc), 0, 0);
  SI  l = (N(pages)&1)==0? odd: even;
  SI  h = paper? height: (lb->h ()+ top+ bot);
  box pb= page_box (path (), lb, as_string (N(pages)+1),
		    width, h, l, top, top+ text_height,
		    make_header(), make_footer(), head_sep, foot_sep);

  cur_top -= h;
  pages << pb;
}

/******************************************************************************
* Typesetting a page
******************************************************************************/

box
pager_rep::make_header () {
  if (!show_hf) return empty_box (decorate ());
  env->write (PAGE_NR, as_string (N(pages)+1));
  string which= (N(pages)&1)==0? PAGE_ODD_HEADER: PAGE_EVEN_HEADER;
  if (style [PAGE_THIS_HEADER] != "") which= PAGE_THIS_HEADER;
  box b= typeset_as_concat (env, tree (PARAGRAPH, style[which]), decorate());
  style (PAGE_THIS_HEADER) = "";
  return b;
}

box
pager_rep::make_footer () {
  if (!show_hf) return empty_box (decorate ());
  env->write (PAGE_NR, as_string (N(pages)+1));
  string which= (N(pages)&1)==0? PAGE_ODD_FOOTER: PAGE_EVEN_FOOTER;
  if (style [PAGE_THIS_FOOTER] != "") which= PAGE_THIS_FOOTER;
  box b= typeset_as_concat (env, tree (PARAGRAPH, style[which]), decorate());
  style (PAGE_THIS_FOOTER) = "";
  return b;
}

box
pager_rep::make_pages () {
  array<int> breaks;
  if (paper) breaks= page_breaks (l, text_height);
  else { breaks= array<int> (2); breaks[0]= 0; breaks[1]= N(l); }

  int i;
  for (i=0; i<N(breaks)-1; i++) {
    int j, start= breaks[i], end= breaks[i+1];
    start_page ();
    for (j=start; j<end; j++) print (l[j]);
    bool newpage=
      (l[j-1]->type == PAGE_CONTROL_ITEM) &&
      (l[j-1]->t == tree (FORMAT, NEW_PAGE));
    end_page (newpage || (i==N(breaks)-2));
  }

  array<SI> spc (N(pages));
  for (i=0; i<N(pages); i++) spc[i]=0;

  return move_box (path (), stack_box (path (), pages, spc), 0, 0);
}

#endmodule // code_pager
