
/******************************************************************************
* MODULE     : skeleton.gen.cc
* DESCRIPTION: Line breaking facility for paragraphs
* COPYRIGHT  : (C) 1999  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#module skeleton
#define no_tree_converter<insertion>

class insertion_rep;
class insertion {
#import concrete (insertion, insertion_rep)
  inline insertion ();
  inline insertion (tree type, path begin, path end);
  inline bool operator == (insertion ins);
  inline bool operator != (insertion ins);
  friend ostream& operator << (ostream& out, insertion ins);
};

#import array (insertion)
typedef array<insertion> insertions;
#import array (insertions)
typedef array<insertions> skeleton;

struct insertion_rep: concrete_struct {
  tree      type;   // type of insertion
  path      begin;  // begin location in array of page_items
  path      end;    // end location in array of page_items
  skeleton  sub;    // possible subpage breaks (used for multicolumns)

  inline insertion_rep () {}
  inline insertion_rep (tree type2, path begin2, path end2):
    type (type2), begin (begin2), end (end2) {}
};

#import code_concrete (insertion, insertion_rep)
#import code_array (insertion)
#import code_array (insertions)

inline
insertion::insertion () {
  rep= new insertion_rep;
}

inline
insertion::insertion (tree type, path begin, path end) {
  rep= new insertion_rep (type, begin, end);
}

inline bool
insertion::operator == (insertion ins) {
  return
    (rep->type  == ins->type ) &&
    (rep->begin == ins->begin) &&
    (rep->end   == ins->end  ) &&
    (rep->sub   == ins->sub  );
}

inline bool
insertion::operator != (insertion ins) {
  return
    (rep->type  != ins->type ) ||
    (rep->begin != ins->begin) ||
    (rep->end   != ins->end  ) ||
    (rep->sub   != ins->sub  );
}

#endmodule // skeleton

#module code_skeleton
#import skeleton

ostream&
operator << (ostream& out, insertion ins) {
  out << ins->type << " insertion [ " << ins->begin << "--" << ins->end;
  if (N(ins->sub)>0) out << "; " << ins->sub;
  return out << " ]";
}

#endmodule // code_skeleton
